<?php

namespace Drupal\easy_entity_field\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class EasyEntityFieldSettingsForm.
 */
class EasyEntityFieldSettingsForm extends FormBase {

  /**
   * The Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The router builder.
   *
   * @var \Drupal\Core\Routing\RouteBuilderInterface
   */
  protected $routerBuilder;

  /**
   * The Cache Render.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheRender;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RouteBuilderInterface $router_builder, CacheBackendInterface $cache_render) {
    $this->entityTypeManager = $entity_type_manager;
    $this->routerBuilder = $router_builder;
    $this->cacheRender = $cache_render;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('router.builder'),
      $container->get('cache.render')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'easy_entity_field_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $all_entity_types = $this->entityTypeManager->getDefinitions();
    $entity_types = [];
    foreach ($all_entity_types as $entity_type) {
      if ($entity_type instanceof ContentEntityTypeInterface) {
        $entity_types[$entity_type->id()] = $entity_type;
      }
    }

    $form['entity_types'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Entity type Name'),
        $this->t('status'),
        $this->t('Select base route'),
        $this->t('Manage'),
      ],
      '#tree' => TRUE,
    ];

    foreach ($entity_types as $entity_type_id => $entity_type) {
      if ($entity_type_id == 'file') {
        continue;
      }
      $settings = $this->configFactory()->getEditable("easy_entity_field.settings.{$entity_type_id}");

      $form['entity_types'][$entity_type_id]['label'] = [
        '#markup' => $this->t('@label', ['@label' => $entity_type->getLabel()]),
      ];

      $form['entity_types'][$entity_type_id]['status'] = [
        '#type' => 'checkbox',
        '#default_value' => $settings->get('status') ?? '',
      ];

      if ($entity_type->getProvider() != 'easy_entity' && $entity_type->getBundleEntityType() && $this->getLocalTasksRouteName($entity_type)) {
        $form['entity_types'][$entity_type_id]['base_route'] = [
          '#type' => 'radios',
          '#options' => ['' => $this->t('None')] + $this->getLocalTasksRouteName($entity_type),
          '#default_value' => $settings->get('base_route') ?? '',
        ];
      }
      else {
        $form['entity_types'][$entity_type_id]['base_route'] = [
          '#type' => 'value',
          '#value' => '',
        ];
      }
      if ($settings->get('status')) {
        $form['entity_types'][$entity_type_id]['manage_base_fields'] = [
          '#type' => 'link',
          '#title' => 'Manage Base fields',
          '#url' => Url::fromRoute("entity.{$entity_type_id}.base_field", [], ['attributes' => ['target' => '_blank']]),
        ];
      }

    }

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    foreach ($form_state->getValue('entity_types') as $entity_type_id => $values) {
      $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
      if ($entity_type->getBundleEntityType() && $values['status']) {
        if ($entity_type->getProvider() != 'easy_entity' && $this->getLocalTasksRouteName($entity_type) && empty($values['base_route'])) {
          $form_state->setErrorByName(
            'base_route',
            $this->t("Please select the default basic route of the local task for the entity type: %entity_type_label.", ['%entity_type_label' => $entity_type->getLabel()])
          );
        }
      }
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->cleanValues();

    foreach ($form_state->getValue('entity_types') as $entity_type_id => $values) {
      if ($values['status']) {
        $settings = $this->configFactory()->getEditable("easy_entity_field.settings.{$entity_type_id}");
        $settings->setData($values)->save();
      }
      else {
        $this->configFactory()->getEditable("easy_entity_field.settings.{$entity_type_id}")->delete();
      }
    }

    drupal_flush_all_caches();
  }

  protected function getLocalTasksRouteName(EntityTypeInterface $entity_type) {
    $route_name = "entity.{$entity_type->getBundleEntityType()}.collection";

    /** @var \Drupal\Core\Menu\LocalTaskManagerInterface $local_tasks */
    $local_tasks = \Drupal::service('plugin.manager.menu.local_task')->getLocalTasksForRoute($route_name);
    if (!isset($local_tasks[0])) {
      return NULL;
    }

    $local_task_route = [];
    foreach ($local_tasks[0] as $local_task) {
      if ($local_task->getRouteName() != "entity.{$entity_type->id()}.base_field") {
        $local_task_route[$local_task->getRouteName()] = $local_task->getRouteName();
      }
    }

    return $local_task_route;
  }

}
