<?php

namespace Drupal\easy_entity_field\Routing;

use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Define the route subscriber.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a RouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */

  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */

  protected function alterRoutes(RouteCollection $collection) {
    $entity_types = $this->entityTypeManager->getDefinitions();
    foreach ($entity_types as $entity_type) {
      if (!$entity_type instanceof ContentEntityTypeInterface) {
        continue;
      }
      $entity_type_id = $entity_type->id();
      if (\Drupal::configFactory()->getEditable("easy_entity_field.settings.{$entity_type_id}")->get('status')) {
        if (!empty($entity_type->getBundleEntityType())) {
          $base_route_name = "entity.{$entity_type->getBundleEntityType()}.collection";
        }
        else {
          $base_route_name = $entity_type->get('field_ui_base_route');
        }

        if (!isset($base_route_name)) {
          continue;
        }

        $entity_route = $collection->get($base_route_name);

        if (!isset($entity_route)) {
          continue;
        }

        $entity_path = $entity_route->getPath();
        $entity_options = $entity_route->getOptions();

        $defaults = [
          'entity_type_id' => $entity_type_id,
          'target_entity_type_id' => $entity_type_id,
        ];

        $entity_options['parameters'][$entity_type_id] = [
          'type' => 'entity:' . $entity_type_id,
        ];

        $entity_options['_field_ui'] = TRUE;

        $route = new Route("$entity_path/base-field");
        $route->setDefaults(
          [
            '_entity_list' => 'easy_entity_field',
            '_title' => 'Base fields',
          ] + $defaults
        )->setRequirements([
          '_permission' => 'administer ' . $entity_type_id . ' base fields',
        ])->setOptions($entity_options);
        $collection->add("entity.{$entity_type_id}.base_field", $route);


        $route = new Route("$entity_path/base-field/add");
        $route->setDefaults(
          [
            '_title' => 'Add base field',
            '_form' => '\Drupal\easy_entity_field\Form\EasyEntityFieldAddForm',
          ] + $defaults
        )->setRequirements([
          '_permission' => 'administer ' . $entity_type_id . ' base fields',
        ])->setOptions($entity_options);
        $collection->add("entity.{$entity_type_id}.base_field.add", $route);

        $route = new Route("$entity_path/base-field/{easy_entity_field}/storage");
        $route->setDefaults(
          [
            '_title' => 'Edit base field storage',
            '_entity_form' => 'easy_entity_field.storage',
          ] + $defaults
        )->setRequirements([
          '_permission' => 'administer ' . $entity_type_id . ' base fields',
        ])->setOptions($entity_options);
        $collection->add("entity.{$entity_type_id}.base_field.storage", $route);

        $route = new Route("$entity_path/base-field/{easy_entity_field}");
        $route->setDefaults(
          [
            '_title' => 'Edit base field',
            '_entity_form' => 'easy_entity_field.edit',
          ] + $defaults
        )->setRequirements([
          '_permission' => 'administer ' . $entity_type_id . ' base fields',
        ])->setOptions($entity_options);
        $collection->add("entity.{$entity_type_id}.base_field.edit", $route);

        $route = new Route("$entity_path/base-field/{easy_entity_field}/delete");
        $route->setDefaults(
          [
            '_title' => 'Delete base field',
            '_entity_form' => 'easy_entity_field.delete',
          ] + $defaults
        )->setRequirements([
          '_permission' => 'administer ' . $entity_type_id . ' base fields',
        ])->setOptions($entity_options);
        $collection->add("entity.{$entity_type_id}.base_field.delete", $route);
        
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = parent::getSubscribedEvents();
    $events[RoutingEvents::ALTER] = ['onAlterRoutes', -100];
    return $events;
  }

}

